@@ -7,7 +7,7 @@ import click
 
 from yojenkins.cli import cli_utility as cu
 from yojenkins.cli.cli_utility import log_to_history
-from yojenkins.utility.utility import wait_for_build_and_follow_logs
+from yojenkins.utility.utility import is_complete_build_url, wait_for_build_and_follow_logs
 from yojenkins.yo_jenkins.status import Status
 
 # Getting the logger reference
@@ -15,7 +15,7 @@ logger = logging.getLogger()
 
 
 def _verify_build_url_get_job_format(build_url: str, job: str) -> bool:
-    """Utility function to verify buld url and determine if job is URL or name
+    """Utility function to verify build url and determine if job is URL or name
 
     Args:
         build_url: Build URL
@@ -46,7 +46,9 @@ def info(profile: str, token: str, job: str, number: int, url: str, latest: bool
         url:     The build url to get info on
         latest:  Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -76,7 +78,9 @@ def status(profile: str, token: str, job: str, number: int, url: str, latest: bo
         url: The build url to get info on
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -121,7 +125,9 @@ def abort(profile: str, token: str, job: str, number: int, url: str, latest: boo
         url: The build url to get info on
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -149,7 +155,9 @@ def delete(profile: str, token: str, job: str, number: int, url: str, latest: bo
         url: The build url to get info on
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -178,7 +186,9 @@ def stages(profile: str, token: str, opt_list: bool, job: str, number: int, url:
         url: The build URL
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -211,7 +221,9 @@ def logs(profile: str, token: str, job: str, number: int, url: str, latest: bool
         download_dir: Option to download the log to a directory
         follow: Option to follow the log
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -252,7 +264,9 @@ def browser(profile: str, token: str, job: str, number: int, url: str, latest: b
         url: The build url to get info on
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -280,7 +294,9 @@ def monitor(profile: str, token: str, job: str, number: int, url: str, latest: b
         latest: Option to get the latest build
         sound: Option to play a sound when the build status changes
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -309,7 +325,9 @@ def parameters(profile: str, token: str, opt_list: bool, job: str, number: int,
         url: The build URL
         latest: Option to get the latest build
     """
-    if job and not number and not latest:
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -339,7 +357,10 @@ def rebuild(profile: str, token: str, job: str, number: int, url: str, latest: b
         latest:  Option to get the latest build
         follow_logs: Waits for the job build, then follows resulting logs
     """
-    if job and not number and not latest:
+
+    if url is None and job and is_complete_build_url(job):
+        url, job = job, None
+    elif job and not number and not latest:
         click.echo(
             click.style('INPUT ERROR: For job, either specify --number or --latest. See --help',
                         fg='bright_red',
@@ -357,3 +378,23 @@ def rebuild(profile: str, token: str, job: str, number: int, url: str, latest: b
         click.secho(f'success. queue number: {data}', fg='bright_green', bold=True)
         return
     wait_for_build_and_follow_logs(yj_obj, data)
+
+
+@log_to_history
+def diff(profile: str, token: str, build_url_1: str, build_url_2: str, logs: bool, char_ignore: int, no_color: bool,
+         diff_only: bool, diff_guide: bool) -> None:
+    """Get the diff comparison for two builds
+
+    Args:
+        profile:     The profile/account to use
+        token:       API Token for Jenkins server
+        build_url_1: First build for comparison
+        build_url_2: Second build for comparison
+        logs:        Compare build logs
+        char_ignore: Number of characters to ignore at start of each line
+        no_color:    Output diff with no color
+        diff_only:   Only show the lines that have changed
+        diff_guide:  Show diff guide, showing where exactly difference is in line
+    """
+    yj_obj = cu.config_yo_jenkins(profile, token)
+    yj_obj.build.diff(build_url_1, build_url_2, logs, char_ignore, no_color, diff_only, diff_guide)
