@@ -7,6 +7,7 @@ from click_help_colors import HelpColorsCommand
 from yojenkins.__main__ import auth
 from yojenkins.cli import cli_auth, cli_decorators
 from yojenkins.cli.cli_utility import set_debug_log_level
+from yojenkins.utility.utility import translate_kwargs
 
 
 @auth.command(short_help='\tConfigure authentication',
@@ -48,32 +49,33 @@ def configure(debug, auth_file):
 @click.option('--server-base-url', type=str, required=False, help='Server base URL address')
 @click.option('--username', type=str, required=False, help='Account username')
 @click.option('--password', type=str, required=False, help='Account password')
-def token(debug, profile, token_name, server_base_url, username, password):
+def token(debug, **kwargs):
     """Generate authentication API token
 
     NOTE: To generate an API token and automatically add it to the
     an existing authentication profile, use the --profile option.
     """
     set_debug_log_level(debug)
-    cli_auth.token(profile, token_name, server_base_url, username, password)
+    cli_auth.token(**translate_kwargs(kwargs))
 
 
 @auth.command(short_help='\tShow the local credentials profiles')
 @cli_decorators.debug
 @cli_decorators.format_output
-def show(debug, pretty, yaml, xml, toml):
+def show(debug, **kwargs):
     """Show the local credentials profiles"""
     set_debug_log_level(debug)
-    cli_auth.show(pretty, yaml, xml, toml)
+    cli_auth.show(**translate_kwargs(kwargs))
 
 
 @auth.command(short_help='\tCheck if credentials can authenticate')
 @cli_decorators.debug
 @cli_decorators.profile
-def verify(debug, profile):
+def verify(debug, **kwargs):
     """Check if credentials can authenticate"""
     set_debug_log_level(debug)
-    cli_auth.verify(profile)
+    del kwargs['token']
+    cli_auth.verify(**translate_kwargs(kwargs))
 
 
 @auth.command(short_help='\tWipe all credentials for this device')
@@ -88,7 +90,7 @@ def wipe(debug):
 @cli_decorators.debug
 @cli_decorators.format_output
 @cli_decorators.profile
-def user(debug, pretty, yaml, xml, toml, profile):
+def user(debug, **kwargs):
     """Show current user information"""
     set_debug_log_level(debug)
-    cli_auth.user(pretty, yaml, xml, toml, profile)
+    cli_auth.user(**translate_kwargs(kwargs))
