@@ -15,7 +15,7 @@ from yojenkins.utility.utility import translate_kwargs
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=bool, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def info(ctx, debug, **kwargs):
@@ -32,7 +32,7 @@ def info(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number', metavar='INT')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=bool, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def status(ctx, debug, **kwargs):
@@ -50,7 +50,7 @@ def status(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def abort(ctx, debug, **kwargs):
@@ -67,7 +67,7 @@ def abort(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def delete(ctx, debug, **kwargs):
@@ -87,7 +87,7 @@ def delete(ctx, debug, **kwargs):
 @cli_decorators.list
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def stages(ctx, debug, **kwargs):
@@ -104,7 +104,7 @@ def stages(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.option('--tail', type=float, required=False, help='Last of logs. If < 1 then %, else number of lines')
 @click.option('-dd',
@@ -145,7 +145,7 @@ def logs(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def browser(ctx, debug, **kwargs):
@@ -163,7 +163,7 @@ def browser(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.option('-s', '--sound', type=bool, required=False, is_flag=True, help='Enable sound effects')
 @click.pass_context
@@ -184,7 +184,7 @@ def monitor(ctx, debug, **kwargs):
 @cli_decorators.list
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.pass_context
 def parameters(ctx, debug, **kwargs):
@@ -204,7 +204,7 @@ def parameters(ctx, debug, **kwargs):
 @cli_decorators.profile
 @click.argument('job', nargs=1, type=str, required=False)
 @click.option('-n', '--number', type=int, required=False, help='Build number')
-@click.option('-u', '--url', type=str, required=False, help='Build URL (No job info needed)')
+@click.option('-u', '--url', type=str, required=False, help='Flexible build URL (No job info needed)')
 @click.option('--latest', type=str, required=False, is_flag=True, help='Latest build (Replaces --number)')
 @click.option('--follow-logs',
               type=bool,
@@ -222,3 +222,51 @@ def rebuild(ctx, debug, **kwargs):
         cli_build.rebuild(**translate_kwargs(kwargs))
     else:
         click.echo(ctx.get_help())
+
+
+@build.command(short_help='\tFind difference between two builds')
+@cli_decorators.debug
+@cli_decorators.profile
+@click.argument('build-url-1', nargs=1, type=str, required=True)
+@click.argument('build-url-2', nargs=1, type=str, required=True)
+# @click.option('--type',
+#               type=click.Choice(['info', 'logs'], case_sensitive=False),
+#               default="info",
+#               show_default=True,
+#               required=False,
+#               help='Type of diff comparison')
+@click.option('--logs', type=bool, default=False, required=False, is_flag=True, help='Build logs diff')
+@click.option('--char-ignore',
+              default=0,
+              type=click.IntRange(0),
+              required=False,
+              help='Number of characters to ignore at line start')
+@click.option('--no-color', type=bool, default=False, required=False, is_flag=True, help='Show output without color')
+@click.option('--diff-only',
+              type=bool,
+              default=False,
+              required=False,
+              is_flag=True,
+              help='Show only lines that are different')
+@click.option('--diff-guide',
+              type=bool,
+              default=False,
+              required=False,
+              is_flag=True,
+              help='Show where the difference is in line')
+# @click.option('--stats-only', type=bool, default=False, required=False, is_flag=True, help='Show diff stats only')
+def diff(debug, **kwargs):
+    """Get the diff comparison for two builds (info, logs)
+
+    EXAMPLES:
+
+    \b
+    - yojenkins build diff "myFolder/myJob/4" "myFolder/myJob/5"
+    - yojenkins build diff "myJob/4/console" "myJob/5" --logs --diff-guide
+    - yojenkins build diff "myJob/5/" "yourJob/8" --diff-only
+    - yojenkins build diff "myJob/2/" "youJob/2/" --logs --char-ignore 40
+    - yojenkins build diff "myJob/5/" "yourJob/8" --stats-only
+
+    """
+    set_debug_log_level(debug)
+    cli_build.diff(**translate_kwargs(kwargs))
